package com.wowza.wms.plugin.test.ws;

import javax.jws.*;
import javax.xml.ws.*;

import com.wowza.wms.application.*;
import com.wowza.wms.logging.*;
import com.wowza.wms.server.*;
import com.wowza.wms.vhost.*;

import edu.emory.mathcs.backport.java.util.concurrent.locks.*;

@WebService

public class WowzaServerWebService
{
	private Endpoint endpoint = null;
	
	public static final String DEFAULT_VHOST = "_defaultVHost_";
	public static final String DEFAULT_APPLICATION = "_defapp_";
	public static final String DEFAULT_APPINSTANCE = "_definst_";

	public static class AppConextName
	{
		String vhostName = null;
		String appName = null;
		String appInstName = null;
		
		public AppConextName()
		{
		}
		
		public AppConextName(String fullname, boolean startWithDefaults)
		{
			if (startWithDefaults)
			{
				vhostName = DEFAULT_VHOST;
				appName = DEFAULT_APPLICATION;
				appInstName = DEFAULT_APPINSTANCE;
			}

			int qloc = fullname.indexOf(":");
			if (qloc >= 0)
			{
				vhostName = fullname.substring(0, qloc);
				fullname = fullname.substring(qloc+1);
			}
			else
				vhostName = DEFAULT_VHOST;
			
			if (fullname.length() > 0)
			{
				appName = fullname;
				int sloc = fullname.indexOf("/");
				if (sloc >= 0)
				{
					appName = fullname.substring(0, sloc);
					appInstName = fullname.substring(sloc+1);
				}
			}
		}
		
		String getObjName()
		{
			String ret = "";
			
			while(true)
			{
				if (vhostName == null)
					break;
				
				ret += "vHosts=VHosts,vHostName="+vhostName;
				
				if (appName == null)
					break;
				
				ret += ",applications=Applications,applicationName="+appName;
				
				if (appInstName == null)
					break;
				
				ret += ",applicationInstances=ApplicationInstances,applicationInstanceName="+appInstName;
				break;
			}
									
			return ret;
		}
	}

	public WowzaServerWebService()
	{
		
	}
		
	public void bind(String host, int port, String path)
	{
		String endpointURL = "http://"+host+":"+port+"/"+path;
		try
		{
			endpoint = Endpoint.publish(endpointURL, this);
			WMSLoggerFactory.getLogger(null).info("WowzaServerWebService.bind: "+endpointURL);
		}
		catch (Exception e)
		{
			WMSLoggerFactory.getLogger(null).error("WowzaServerWebService.host: "+e.toString());
		}
	}
	
	public void shutdown()
	{
		WMSLoggerFactory.getLogger(null).info("WowzaServerWebService.shutdown");
		if (endpoint != null)
			endpoint.stop();
		endpoint = null;
	}

	@WebMethod
	public String getServerVersion()
	{
		IServer server = Server.getInstance();
		WMSLoggerFactory.getLogger(null).info("WowzaServerWebService.getServerVersion: "+server.getVersion());
		return server.getVersion();
	}
	
	@WebMethod
	public void startAppInstance(String appInstanceStr)
	{
		WMSLoggerFactory.getLogger(null).info("WowzaServerWebService.startAppInstance: appInstance:"+appInstanceStr);

		AppConextName context = new AppConextName(appInstanceStr, true);
		
		try
		{
			while(true)
			{
				IVHost vhost = VHostSingleton.getInstance(context.vhostName);
				if (vhost == null)
				{
					WMSLoggerFactory.getLogger(null).warn("WowzaServerWebService.stopAppInstance: vhost missing: "+context.vhostName);
					break;
				}
				
				vhost.startApplicationInstance(context.appName, context.appInstName);
				break;
			}
		}
		catch(Exception e)
		{
			WMSLoggerFactory.getLogger(null).error("WowzaServerWebService.startAppInstance: "+e.toString());
		}		
	}

	@WebMethod
	public void stopAppInstance(String appInstanceStr)
	{
		WMSLoggerFactory.getLogger(null).info("WowzaServerWebService.stopAppInstance: appInstance:"+appInstanceStr);

		AppConextName context = new AppConextName(appInstanceStr, true);
		
		try
		{
			while(true)
			{
				IVHost vhost = VHostSingleton.getInstance(context.vhostName);
				if (vhost == null)
				{
					WMSLoggerFactory.getLogger(null).warn("WowzaServerWebService.stopAppInstance: vhost missing: "+context.vhostName);
					break;
				}
				
				IApplication application = vhost.getApplication(context.appName);
				if (application == null)
				{
					WMSLoggerFactory.getLogger(null).warn("WowzaServerWebService.stopAppInstance: application missing: "+context.appName);
					break;
				}
				
				application.shutdownAppInstance(context.appInstName);
				break;
			}
		}
		catch(Exception e)
		{
			WMSLoggerFactory.getLogger(null).error("WowzaServerWebService.stopAppInstance: "+e.toString());
		}		
	}

	@WebMethod
	public void startMediaCasterStream(String appInstanceStr, String streamName, String mediaCasterType)
	{
		WMSLoggerFactory.getLogger(null).info("WowzaServerWebService.startMediaCasterStream: appInstance:"+appInstanceStr+" streamName:"+streamName+" mediaCasterType:"+mediaCasterType);

		AppConextName context = new AppConextName(appInstanceStr, true);
		
		try
		{
			while(true)
			{
				IVHost vhost = VHostSingleton.getInstance(context.vhostName);
				if (vhost == null)
				{
					WMSLoggerFactory.getLogger(null).warn("WowzaServerWebService.startMediaCasterStream: vhost missing: "+context.vhostName);
					break;
				}
				
				vhost.startApplicationInstance(context.appName, context.appInstName);
				
				WMSReadWriteLock lock = vhost.getApplicationLock();
				
				IApplication application = null;
				IApplicationInstance appInstance = null;
				try
				{
					lock.writeLock().lock();
					application = vhost.getApplication(context.appName);
					if (application == null)
						WMSLoggerFactory.getLogger(null).warn("WowzaServerWebService.startMediaCasterStream: application missing: "+context.appName);
					else
						appInstance = application.getAppInstance(context.appInstName);
					
					if (appInstance == null)
						WMSLoggerFactory.getLogger(null).warn("WowzaServerWebService.startMediaCasterStream: appInstance missing: "+context.appInstName);
				}
				catch(Exception e)
				{	
				}
				finally
				{
					lock.writeLock().unlock();
				}
				
				if (appInstance == null)
					break;
				
				appInstance.startMediaCasterStream(streamName, mediaCasterType);
				break;
			}
		}
		catch(Exception e)
		{
			WMSLoggerFactory.getLogger(null).error("WowzaServerWebService.startMediaCasterStream: "+e.toString());
		}		
	}

	@WebMethod
	public void stopMediaCasterStream(String appInstanceStr, String streamName)
	{
		WMSLoggerFactory.getLogger(null).info("WowzaServerWebService.stopMediaCasterStream: appInstance:"+appInstanceStr+" streamName:"+streamName);

		AppConextName context = new AppConextName(appInstanceStr, true);
		
		try
		{
			while(true)
			{
				IVHost vhost = VHostSingleton.getInstance(context.vhostName);
				if (vhost == null)
				{
					WMSLoggerFactory.getLogger(null).warn("WowzaServerWebService.stopMediaCasterStream: vhost missing: "+context.vhostName);
					break;
				}
				
				vhost.startApplicationInstance(context.appName, context.appInstName);
				
				WMSReadWriteLock lock = vhost.getApplicationLock();
				
				IApplication application = null;
				IApplicationInstance appInstance = null;
				try
				{
					lock.writeLock().lock();
					application = vhost.getApplication(context.appName);
					if (application == null)
						WMSLoggerFactory.getLogger(null).warn("WowzaServerWebService.startMediaCasterStream: application missing: "+context.appName);
					else
						appInstance = application.getAppInstance(context.appInstName);
					
					if (appInstance == null)
						WMSLoggerFactory.getLogger(null).warn("WowzaServerWebService.startMediaCasterStream: appInstance missing: "+context.appInstName);
				}
				catch(Exception e)
				{	
				}
				finally
				{
					lock.writeLock().unlock();
				}
				
				if (appInstance == null)
					break;
				
				appInstance.stopMediaCasterStream(streamName);
				break;
			}
		}
		catch(Exception e)
		{
			WMSLoggerFactory.getLogger(null).error("WowzaServerWebService.stopMediaCasterStream: "+e.toString());
		}		
	}
}
